# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Configuration for a Vulnerability Audit
  # type: object
  #
  class Adm::Models::VulnerabilityAuditConfiguration
    # The minimum CVSS V2 score for a vulnerability to be considered by the audit as failed.
    # @return [Float]
    attr_accessor :max_permissible_cvss_v2_score

    # The minimum CVSS V3 score for a vulnerability to be considered by the audit as failed.
    # @return [Float]
    attr_accessor :max_permissible_cvss_v3_score

    # Pattern to exclude artifacts from audit.
    # @return [Array<String>]
    attr_accessor :exclusions

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'max_permissible_cvss_v2_score': :'maxPermissibleCvssV2Score',
        'max_permissible_cvss_v3_score': :'maxPermissibleCvssV3Score',
        'exclusions': :'exclusions'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'max_permissible_cvss_v2_score': :'Float',
        'max_permissible_cvss_v3_score': :'Float',
        'exclusions': :'Array<String>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [Float] :max_permissible_cvss_v2_score The value to assign to the {#max_permissible_cvss_v2_score} property
    # @option attributes [Float] :max_permissible_cvss_v3_score The value to assign to the {#max_permissible_cvss_v3_score} property
    # @option attributes [Array<String>] :exclusions The value to assign to the {#exclusions} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.max_permissible_cvss_v2_score = attributes[:'maxPermissibleCvssV2Score'] if attributes[:'maxPermissibleCvssV2Score']
      self.max_permissible_cvss_v2_score = 0.0 if max_permissible_cvss_v2_score.nil? && !attributes.key?(:'maxPermissibleCvssV2Score') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :maxPermissibleCvssV2Score and :max_permissible_cvss_v2_score' if attributes.key?(:'maxPermissibleCvssV2Score') && attributes.key?(:'max_permissible_cvss_v2_score')

      self.max_permissible_cvss_v2_score = attributes[:'max_permissible_cvss_v2_score'] if attributes[:'max_permissible_cvss_v2_score']
      self.max_permissible_cvss_v2_score = 0.0 if max_permissible_cvss_v2_score.nil? && !attributes.key?(:'maxPermissibleCvssV2Score') && !attributes.key?(:'max_permissible_cvss_v2_score') # rubocop:disable Style/StringLiterals

      self.max_permissible_cvss_v3_score = attributes[:'maxPermissibleCvssV3Score'] if attributes[:'maxPermissibleCvssV3Score']
      self.max_permissible_cvss_v3_score = 0.0 if max_permissible_cvss_v3_score.nil? && !attributes.key?(:'maxPermissibleCvssV3Score') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :maxPermissibleCvssV3Score and :max_permissible_cvss_v3_score' if attributes.key?(:'maxPermissibleCvssV3Score') && attributes.key?(:'max_permissible_cvss_v3_score')

      self.max_permissible_cvss_v3_score = attributes[:'max_permissible_cvss_v3_score'] if attributes[:'max_permissible_cvss_v3_score']
      self.max_permissible_cvss_v3_score = 0.0 if max_permissible_cvss_v3_score.nil? && !attributes.key?(:'maxPermissibleCvssV3Score') && !attributes.key?(:'max_permissible_cvss_v3_score') # rubocop:disable Style/StringLiterals

      self.exclusions = attributes[:'exclusions'] if attributes[:'exclusions']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        max_permissible_cvss_v2_score == other.max_permissible_cvss_v2_score &&
        max_permissible_cvss_v3_score == other.max_permissible_cvss_v3_score &&
        exclusions == other.exclusions
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [max_permissible_cvss_v2_score, max_permissible_cvss_v3_score, exclusions].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
